#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>
#include <Wire.h>
#include <TinyGPS++.h>
#include <HardwareSerial.h>

#define RXD2 3
#define TXD2 1

HardwareSerial neogps(2);
TinyGPSPlus gps;



BLEServer* pServer = NULL;
BLECharacteristic* pCharacteristic = NULL;
BLEDescriptor *pDescr;
BLE2902 *pBLE2902;

bool deviceConnected = false;
bool oldDeviceConnected = false;
uint32_t value = 0;

#define SERVICE_UUID        "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

class MyServerCallbacks: public BLEServerCallbacks {
void onConnect(BLEServer* pServer) {deviceConnected = true;};
void onDisconnect(BLEServer* pServer) {deviceConnected = false;}};

void setup() 
{
  BluetoothSetup();
  neogps.begin(9600, SERIAL_8N1, RXD2, TXD2);
  //neogps.print("GPS Module Initialized");
  delay(3000);
}

void loop() 
{
        if (deviceConnected) {
              pCharacteristic->setValue(value);
              pCharacteristic->notify();
              value++;

              boolean newData = false;
        for (unsigned long start = millis(); millis() - start < 1000;)
        {
          while (neogps.available())
          {
            if (gps.encode(neogps.read()))
            {
              newData = true;
            }
          }
        }

        if (gps.location.isValid())
          {
            String Latatude = String(gps.location.lat(),4);
            String Longitude = String(gps.location.lng(),4);

            String Result = Latatude + ";" + Longitude;
            
            neogps.println(Result);
          }
          else
          {
            //neogps.println("Finding satellites");
            delay(5000);
          }

        delay(1000);
    }
    // disconnecting
    if (!deviceConnected && oldDeviceConnected) {
        delay(500); // give the bluetooth stack the chance to get things ready
        pServer->startAdvertising(); // restart advertising
        //Serial.println("start advertising");
        oldDeviceConnected = deviceConnected;
    }
    // connecting
    if (deviceConnected && !oldDeviceConnected) {
        // do stuff here on connecting
        oldDeviceConnected = deviceConnected;
    }
}

void BluetoothSetup() 
{
  // Create the BLE Device
  BLEDevice::init("TheCircuitBoard");

  // Create the BLE Server
  pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  // Create the BLE Service
  BLEService *pService = pServer->createService(SERVICE_UUID);

  // Create a BLE Characteristic
  pCharacteristic = pService->createCharacteristic(CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_NOTIFY);                   

  // Create a BLE Descriptor
  pDescr = new BLEDescriptor((uint16_t)0x2901);
  pDescr->setValue("A very interesting variable");
  pCharacteristic->addDescriptor(pDescr);
  
  pBLE2902 = new BLE2902();
  pBLE2902->setNotifications(true);
  pCharacteristic->addDescriptor(pBLE2902);

  // Start the service
  pService->start();

  // Start advertising
  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pAdvertising->setScanResponse(false);
  pAdvertising->setMinPreferred(0x0);  // set value to 0x00 to not advertise this parameter
  BLEDevice::startAdvertising();
  //Serial.println("Waiting a client connection to notify...");
}
